import 'package:diameter/components/detail.dart';
import 'package:diameter/components/dialogs.dart';
import 'package:diameter/components/forms.dart';
import 'package:diameter/config.dart';
import 'package:diameter/models/log_event_type.dart';
import 'package:diameter/navigation.dart';
import 'package:flutter/material.dart';

class LogEventTypeDetailScreen extends StatefulWidget {
  static const String routeName = '/log-event-type';
  final LogEventType? logEventType;
  const LogEventTypeDetailScreen({Key? key, this.logEventType})
      : super(key: key);

  @override
  _LogEventTypeDetailScreenState createState() =>
      _LogEventTypeDetailScreenState();
}

class _LogEventTypeDetailScreenState extends State<LogEventTypeDetailScreen> {
  final GlobalKey<FormState> _logEventTypeForm = GlobalKey<FormState>();
  final _valueController = TextEditingController(text: '');
  final _defaultReminderDurationController = TextEditingController(text: '');
  final _notesController = TextEditingController(text: '');
  bool _hasEndTime = false;

  bool _isSaving = false;

  @override
  void initState() {
    super.initState();

    if (widget.logEventType != null) {
      _valueController.text = widget.logEventType!.value;
      _defaultReminderDurationController.text =
          (widget.logEventType!.defaultReminderDuration ?? '').toString();
      _notesController.text = widget.logEventType!.notes ?? '';
      _hasEndTime = widget.logEventType!.hasEndTime;
    }
  }

  void handleSaveAction() async {
    setState(() {
      _isSaving = true;
    });
    if (_logEventTypeForm.currentState!.validate()) {
      bool isNew = widget.logEventType == null;
      LogEventType.put(LogEventType(
        id: widget.logEventType?.id ?? 0,
        value: _valueController.text,
        notes: _notesController.text,
        defaultReminderDuration:
            int.tryParse(_defaultReminderDurationController.text),
        hasEndTime: _hasEndTime,
      ));
      Navigator.pop(context, '${isNew ? 'New' : ''} Log Event Type Saved');
    }
    setState(() {
      _isSaving = false;
    });
  }

  void handleCancelAction() {
    bool isNew = widget.logEventType == null;
    if (showConfirmationDialogOnCancel &&
        ((isNew &&
                (_valueController.text != '' ||
                    int.tryParse(_defaultReminderDurationController.text) !=
                        null ||
                    _notesController.text != '' ||
                    _hasEndTime)) ||
            (!isNew &&
                (_valueController.text != widget.logEventType!.value ||
                    int.tryParse(_defaultReminderDurationController.text) !=
                        widget.logEventType!.defaultReminderDuration ||
                    _notesController.text !=
                        (widget.logEventType!.notes ?? '') ||
                    _hasEndTime != widget.logEventType!.hasEndTime)))) {
      Dialogs.showCancelConfirmationDialog(
        context: context,
        isNew: isNew,
        onSave: handleSaveAction,
      );
    } else {
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    bool isNew = widget.logEventType == null;
    return Scaffold(
      appBar: AppBar(
        title: Text(isNew ? 'New Log Event Type' : widget.logEventType!.value),
      ),
      drawer:
          const Navigation(currentLocation: LogEventTypeDetailScreen.routeName),
      body: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            FormWrapper(
              formState: _logEventTypeForm,
              fields: [
                TextFormField(
                  controller: _valueController,
                  decoration: const InputDecoration(
                    labelText: 'Name',
                    alignLabelWithHint: true,
                  ),
                  validator: (value) {
                    if (value!.trim().isEmpty) {
                      return 'Empty name';
                    }
                    return null;
                  },
                ),
                BooleanFormField(
                  value: _hasEndTime,
                  label: 'has end time',
                  onChanged: (value) {
                    setState(() {
                      _hasEndTime = value;
                    });
                  },
                ),
                TextFormField(
                  controller: _defaultReminderDurationController,
                  keyboardType: const TextInputType.numberWithOptions(),
                  decoration: InputDecoration(
                    labelText: 'Default Reminder Duration',
                    suffixText: ' min',
                    enabled: _hasEndTime,
                  ),
                ),
                TextFormField(
                  controller: _notesController,
                  decoration: const InputDecoration(
                    labelText: 'Notes',
                    alignLabelWithHint: true,
                  ),
                  keyboardType: TextInputType.multiline,
                ),
              ],
            ),
          ],
        ),
      ),
      bottomNavigationBar: DetailBottomRow(
        onCancel: handleCancelAction,
        onSave: _isSaving ? null : handleSaveAction,
      ),
    );
  }
}
